import time

import jwt
from fastapi import Depends, APIRouter, Request

from oic import rndstr
from model.login_oic import LoginRequest, LoginTokenResponse, LoginResponse
from model.wapi.cntl import APIToken
from model.wapi.cntl import Mgr
from model.settings import settings
from . import get_conn
from util.auth import check_auth

router = APIRouter(
    prefix="/api",
    tags=["login_null"]
)
js_mods = ['login_oic']


@router.post('/login')
def api_login(login_request: LoginRequest, request: Request) -> LoginResponse:
    origin = str(request.base_url)

    redirect_uri = origin + settings.oic_redirect_path

    nonce = rndstr()
    state = jwt.encode({
        'stay_logged_in': login_request.stay_logged_in,
        'time': str(int(time.time())),
        'redirect_uri': redirect_uri,
        'nonce': nonce
    }, settings.jwt_secret, algorithm=settings.jwt_algorithm)

    return LoginResponse(login=settings.oic_redirect_path + '?code=dummy', state=state)


@router.get('/oic_create_token')
def oic_create_token(request: Request, conn=Depends(get_conn)) -> LoginTokenResponse:
    db_manager = Mgr(login_name=settings.null_login_name)
    login_token = db_manager.create_session_token(conn, True)  # returns API-Token
    return LoginTokenResponse(token=login_token)


@router.get('/me')
def me_debug(token: APIToken = Depends(check_auth),
             conn=Depends(get_conn)) -> Mgr:
    mgr = Mgr.check_token(conn, token)
    mgr.get_permissions(conn)
    mgr.get_bcds(conn)
    return mgr
